@@ -142,7 +142,7 @@ struct _GstAdapter
   GObject object;
 
   /*< private > */
-  GstQueueArray *bufqueue;
+  GstVecDeque *bufqueue;
   gsize size;
   gsize skip;
   guint count;
@@ -209,7 +209,7 @@ gst_adapter_init (GstAdapter * adapter)
   adapter->dts_at_discont = GST_CLOCK_TIME_NONE;
   adapter->offset_at_discont = GST_BUFFER_OFFSET_NONE;
   adapter->distance_from_discont = 0;
-  adapter->bufqueue = gst_queue_array_new (10);
+  adapter->bufqueue = gst_vec_deque_new (10);
 }
 
 static void
@@ -229,7 +229,7 @@ gst_adapter_finalize (GObject * object)
 
   g_free (adapter->assembled_data);
 
-  gst_queue_array_free (adapter->bufqueue);
+  gst_vec_deque_free (adapter->bufqueue);
 
   GST_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
@@ -262,7 +262,7 @@ gst_adapter_clear (GstAdapter * adapter)
   if (adapter->info.memory)
     gst_adapter_unmap (adapter);
 
-  while ((obj = gst_queue_array_pop_head (adapter->bufqueue)))
+  while ((obj = gst_vec_deque_pop_head (adapter->bufqueue)))
     gst_mini_object_unref (obj);
 
   adapter->count = 0;
@@ -334,11 +334,11 @@ copy_into_unchecked (GstAdapter * adapter, guint8 * dest, gsize skip,
   } else {
     idx = 0;
   }
-  buf = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+  buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
   bsize = gst_buffer_get_size (buf);
   while (G_UNLIKELY (skip >= bsize)) {
     skip -= bsize;
-    buf = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+    buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     bsize = gst_buffer_get_size (buf);
   }
   /* copy partial buffer */
@@ -353,7 +353,7 @@ copy_into_unchecked (GstAdapter * adapter, guint8 * dest, gsize skip,
 
   /* second step, copy remainder */
   while (size > 0) {
-    buf = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+    buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     bsize = gst_buffer_get_size (buf);
     if (G_LIKELY (bsize > 0)) {
       csize = MIN (bsize, size);
@@ -386,16 +386,16 @@ gst_adapter_push (GstAdapter * adapter, GstBuffer * buf)
   adapter->size += size;
 
   /* Note: merging buffers at this point is premature. */
-  if (gst_queue_array_is_empty (adapter->bufqueue)) {
+  if (gst_vec_deque_is_empty (adapter->bufqueue)) {
     GST_LOG_OBJECT (adapter, "pushing %p first %" G_GSIZE_FORMAT " bytes",
         buf, size);
-    gst_queue_array_push_tail (adapter->bufqueue, buf);
+    gst_vec_deque_push_tail (adapter->bufqueue, buf);
     update_timestamps_and_offset (adapter, buf);
   } else {
     /* Otherwise append to the end, and advance our end pointer */
     GST_LOG_OBJECT (adapter, "pushing %p %" G_GSIZE_FORMAT " bytes at end, "
         "size now %" G_GSIZE_FORMAT, buf, size, adapter->size);
-    gst_queue_array_push_tail (adapter->bufqueue, buf);
+    gst_vec_deque_push_tail (adapter->bufqueue, buf);
   }
   ++adapter->count;
 }
@@ -461,7 +461,7 @@ gst_adapter_try_to_merge_up (GstAdapter * adapter, gsize size)
 /**
  * gst_adapter_map:
  * @adapter: a #GstAdapter
- * @size: the number of bytes to map/peek
+ * @size: (in): the number of bytes to map/peek
  *
  * Gets the first @size bytes stored in the @adapter. The returned pointer is
  * valid until the next function is called on the adapter.
@@ -507,7 +507,7 @@ gst_adapter_map (GstAdapter * adapter, gsize size)
 #if 0
   do {
 #endif
-    cur = gst_queue_array_peek_head (adapter->bufqueue);
+    cur = gst_vec_deque_peek_head (adapter->bufqueue);
     skip = adapter->skip;
 
     csize = gst_buffer_get_size (cur);
@@ -568,7 +568,7 @@ gst_adapter_unmap (GstAdapter * adapter)
   g_return_if_fail (GST_IS_ADAPTER (adapter));
 
   if (adapter->info.memory) {
-    GstBuffer *cur = gst_queue_array_peek_head (adapter->bufqueue);
+    GstBuffer *cur = gst_vec_deque_peek_head (adapter->bufqueue);
     GST_LOG_OBJECT (adapter, "unmap memory buffer %p", cur);
     gst_buffer_unmap (cur, &adapter->info);
     adapter->info.memory = NULL;
@@ -581,7 +581,7 @@ gst_adapter_unmap (GstAdapter * adapter)
  * @dest: (out caller-allocates) (array length=size) (element-type guint8):
  *     the memory to copy into
  * @offset: the bytes offset in the adapter to start from
- * @size: the number of bytes to copy
+ * @size: (in): the number of bytes to copy
  *
  * Copies @size bytes of data starting at @offset out of the buffers
  * contained in #GstAdapter into an array @dest provided by the caller.
@@ -648,7 +648,7 @@ gst_adapter_flush_unchecked (GstAdapter * adapter, gsize flush)
   adapter->offset_distance -= adapter->skip;
   adapter->distance_from_discont -= adapter->skip;
 
-  cur = gst_queue_array_peek_head (adapter->bufqueue);
+  cur = gst_vec_deque_peek_head (adapter->bufqueue);
   size = gst_buffer_get_size (cur);
   while (flush >= size) {
     /* can skip whole buffer */
@@ -662,14 +662,14 @@ gst_adapter_flush_unchecked (GstAdapter * adapter, gsize flush)
     --adapter->count;
 
     cur = NULL;
-    gst_buffer_unref (gst_queue_array_pop_head (adapter->bufqueue));
+    gst_buffer_unref (gst_vec_deque_pop_head (adapter->bufqueue));
 
-    if (gst_queue_array_is_empty (adapter->bufqueue)) {
+    if (gst_vec_deque_is_empty (adapter->bufqueue)) {
       GST_LOG_OBJECT (adapter, "adapter empty now");
       break;
     }
     /* there is a new head buffer, update the timestamps */
-    cur = gst_queue_array_peek_head (adapter->bufqueue);
+    cur = gst_vec_deque_peek_head (adapter->bufqueue);
     update_timestamps_and_offset (adapter, cur);
     size = gst_buffer_get_size (cur);
   }
@@ -753,7 +753,7 @@ gst_adapter_get_internal (GstAdapter * adapter, gsize nbytes)
 /**
  * gst_adapter_take:
  * @adapter: a #GstAdapter
- * @nbytes: the number of bytes to take
+ * @nbytes: (in): the number of bytes to take
  *
  * Returns a freshly allocated buffer containing the first @nbytes bytes of the
  * @adapter. The returned bytes will be flushed from the adapter.
@@ -828,7 +828,7 @@ gst_adapter_get_buffer_fast (GstAdapter * adapter, gsize nbytes)
     return NULL;
 
   skip = adapter->skip;
-  cur = gst_queue_array_peek_head (adapter->bufqueue);
+  cur = gst_vec_deque_peek_head (adapter->bufqueue);
 
   if (skip == 0 && gst_buffer_get_size (cur) == nbytes) {
     GST_LOG_OBJECT (adapter, "providing buffer of %" G_GSIZE_FORMAT " bytes"
@@ -837,12 +837,12 @@ gst_adapter_get_buffer_fast (GstAdapter * adapter, gsize nbytes)
     goto done;
   }
 
-  len = gst_queue_array_get_length (adapter->bufqueue);
+  len = gst_vec_deque_get_length (adapter->bufqueue);
 
   for (idx = 0; idx < len && left > 0; idx++) {
     gsize size, cur_size;
 
-    cur = gst_queue_array_peek_nth (adapter->bufqueue, idx);
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx);
     cur_size = gst_buffer_get_size (cur);
     size = MIN (cur_size - skip, left);
 
@@ -980,7 +980,7 @@ gst_adapter_get_buffer (GstAdapter * adapter, gsize nbytes)
   if (G_UNLIKELY (nbytes > adapter->size))
     return NULL;
 
-  cur = gst_queue_array_peek_head (adapter->bufqueue);
+  cur = gst_vec_deque_peek_head (adapter->bufqueue);
   skip = adapter->skip;
   hsize = gst_buffer_get_size (cur);
 
@@ -1020,10 +1020,10 @@ gst_adapter_get_buffer (GstAdapter * adapter, gsize nbytes)
     gsize read_offset = 0;
 
     idx = 0;
-    len = gst_queue_array_get_length (adapter->bufqueue);
+    len = gst_vec_deque_get_length (adapter->bufqueue);
 
     while (idx < len && read_offset < nbytes + adapter->skip) {
-      cur = gst_queue_array_peek_nth (adapter->bufqueue, idx);
+      cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx);
 
       gst_buffer_foreach_meta (cur, foreach_metadata, buffer);
       read_offset += gst_buffer_get_size (cur);
@@ -1054,6 +1054,14 @@ done:
  * The caller needs to explicitly set or unset flags that should be set or
  * unset.
  *
+ * Likewise, no assumptions should be made about timestamps and offset of the
+ * returned buffer. The caller should use gst_adapter_prev_pts(),
+ * gst_adapter_prev_dts(), and gst_adapter_prev_offset() to obtain the relevant
+ * information.
+ *
+ * The returned buffer might not be writable, use gst_buffer_make_writable()
+ * if you need to change e.g. flags or timestamps.
+ *
  * Since 1.6 this will also copy over all GstMeta of the input buffers except
  * for meta with the %GST_META_FLAG_POOLED flag or with the "memory" tag.
  *
@@ -1111,7 +1119,7 @@ gst_adapter_take_list (GstAdapter * adapter, gsize nbytes)
   GST_LOG_OBJECT (adapter, "taking %" G_GSIZE_FORMAT " bytes", nbytes);
 
   while (nbytes > 0) {
-    cur = gst_queue_array_peek_head (adapter->bufqueue);
+    cur = gst_vec_deque_peek_head (adapter->bufqueue);
     skip = adapter->skip;
     cur_size = gst_buffer_get_size (cur);
     hsize = MIN (nbytes, cur_size - skip);
@@ -1160,7 +1168,7 @@ gst_adapter_get_list (GstAdapter * adapter, gsize nbytes)
   skip = adapter->skip;
 
   while (nbytes > 0) {
-    cur = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     cur_size = gst_buffer_get_size (cur);
     hsize = MIN (nbytes, cur_size - skip);
 
@@ -1226,7 +1234,7 @@ gst_adapter_take_buffer_list (GstAdapter * adapter, gsize nbytes)
   buffer_list = gst_buffer_list_new_sized (n_bufs);
 
   while (nbytes > 0) {
-    cur = gst_queue_array_peek_head (adapter->bufqueue);
+    cur = gst_vec_deque_peek_head (adapter->bufqueue);
     skip = adapter->skip;
     cur_size = gst_buffer_get_size (cur);
     hsize = MIN (nbytes, cur_size - skip);
@@ -1283,7 +1291,7 @@ gst_adapter_get_buffer_list (GstAdapter * adapter, gsize nbytes)
   skip = adapter->skip;
 
   while (nbytes > 0) {
-    cur = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     cur_size = gst_buffer_get_size (cur);
     hsize = MIN (nbytes, cur_size - skip);
 
@@ -1314,6 +1322,10 @@ gst_adapter_get_buffer_list (GstAdapter * adapter, gsize nbytes)
  * value that can be supplied to gst_adapter_map() without that function
  * returning %NULL.
  *
+ * Calling gst_adapter_map() with the amount of bytes returned by this function
+ * may require expensive operations (like copying the data into a temporary
+ * buffer) in some cases.
+ *
  * Returns: number of bytes available in @adapter
  */
 gsize
@@ -1328,8 +1340,11 @@ gst_adapter_available (GstAdapter * adapter)
  * gst_adapter_available_fast:
  * @adapter: a #GstAdapter
  *
- * Gets the maximum number of bytes that are immediately available without
- * requiring any expensive operations (like copying the data into a
+ * Gets the maximum number of bytes that can be retrieved in a single map
+ * operation without merging buffers.
+ *
+ * Calling gst_adapter_map() with the amount of bytes returned by this function
+ * will never require any expensive operations (like copying the data into a
  * temporary buffer).
  *
  * Returns: number of bytes that are available in @adapter without expensive
@@ -1355,7 +1370,7 @@ gst_adapter_available_fast (GstAdapter * adapter)
   /* take the first non-zero buffer */
   idx = 0;
   while (TRUE) {
-    cur = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     size = gst_buffer_get_size (cur);
     if (size != 0)
       break;
@@ -1454,7 +1469,7 @@ gst_adapter_dts_at_discont (GstAdapter * adapter)
  *
  * The offset is reset to GST_BUFFER_OFFSET_NONE and the distance is set to 0
  * when the adapter is first created or when it is cleared. This also means that
- * before the first byte with an offset is removed from the adapter, the offset
+ * before the first byte with an offset is added to the adapter, the offset
  * and distance returned are GST_BUFFER_OFFSET_NONE and 0 respectively.
  *
  * Since: 1.10
@@ -1483,7 +1498,7 @@ gst_adapter_prev_offset (GstAdapter * adapter, guint64 * distance)
  *
  * The pts is reset to GST_CLOCK_TIME_NONE and the distance is set to 0 when
  * the adapter is first created or when it is cleared. This also means that before
- * the first byte with a pts is removed from the adapter, the pts
+ * the first byte with a pts is added to the adapter, the pts
  * and distance returned are GST_CLOCK_TIME_NONE and 0 respectively.
  *
  * Returns: The previously seen pts.
@@ -1510,7 +1525,7 @@ gst_adapter_prev_pts (GstAdapter * adapter, guint64 * distance)
  *
  * The dts is reset to GST_CLOCK_TIME_NONE and the distance is set to 0 when
  * the adapter is first created or when it is cleared. This also means that before
- * the first byte with a dts is removed from the adapter, the dts
+ * the first byte with a dts is added to the adapter, the dts
  * and distance returned are GST_CLOCK_TIME_NONE and 0 respectively.
  *
  * Returns: The previously seen dts.
@@ -1538,7 +1553,7 @@ gst_adapter_prev_dts (GstAdapter * adapter, guint64 * distance)
  *
  * The pts is reset to GST_CLOCK_TIME_NONE and the distance is set to 0 when
  * the adapter is first created or when it is cleared. This also means that before
- * the first byte with a pts is removed from the adapter, the pts
+ * the first byte with a pts is added to the adapter, the pts
  * and distance returned are GST_CLOCK_TIME_NONE and 0 respectively.
  *
  * Since: 1.2
@@ -1557,10 +1572,10 @@ gst_adapter_prev_pts_at_offset (GstAdapter * adapter, gsize offset,
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), GST_CLOCK_TIME_NONE);
 
   idx = 0;
-  len = gst_queue_array_get_length (adapter->bufqueue);
+  len = gst_vec_deque_get_length (adapter->bufqueue);
 
   while (idx < len && read_offset < offset + adapter->skip) {
-    cur = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
 
     if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_PTS (cur))) {
       pts = GST_BUFFER_PTS (cur);
@@ -1588,7 +1603,7 @@ gst_adapter_prev_pts_at_offset (GstAdapter * adapter, gsize offset,
  *
  * The dts is reset to GST_CLOCK_TIME_NONE and the distance is set to 0 when
  * the adapter is first created or when it is cleared. This also means that before
- * the first byte with a dts is removed from the adapter, the dts
+ * the first byte with a dts is added to the adapter, the dts
  * and distance returned are GST_CLOCK_TIME_NONE and 0 respectively.
  *
  * Since: 1.2
@@ -1607,10 +1622,10 @@ gst_adapter_prev_dts_at_offset (GstAdapter * adapter, gsize offset,
   g_return_val_if_fail (GST_IS_ADAPTER (adapter), GST_CLOCK_TIME_NONE);
 
   idx = 0;
-  len = gst_queue_array_get_length (adapter->bufqueue);
+  len = gst_vec_deque_get_length (adapter->bufqueue);
 
   while (idx < len && read_offset < offset + adapter->skip) {
-    cur = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+    cur = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
 
     if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DTS (cur))) {
       dts = GST_BUFFER_DTS (cur);
@@ -1680,13 +1695,13 @@ gst_adapter_masked_scan_uint32_peek (GstAdapter * adapter, guint32 mask,
     adapter->scan_offset = 0;
     adapter->scan_entry_idx = G_MAXUINT;
   }
-  buf = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+  buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
   bsize = gst_buffer_get_size (buf);
   while (G_UNLIKELY (skip >= bsize)) {
     skip -= bsize;
     adapter->scan_offset += bsize;
     adapter->scan_entry_idx = idx;
-    buf = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+    buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
     bsize = gst_buffer_get_size (buf);
   }
   /* get the data now */
@@ -1725,7 +1740,7 @@ gst_adapter_masked_scan_uint32_peek (GstAdapter * adapter, guint32 mask,
     adapter->scan_offset += info.size;
     adapter->scan_entry_idx = idx;
     gst_buffer_unmap (buf, &info);
-    buf = gst_queue_array_peek_nth (adapter->bufqueue, idx++);
+    buf = gst_vec_deque_peek_nth (adapter->bufqueue, idx++);
 
     if (!gst_buffer_map (buf, &info, GST_MAP_READ))
       return -1;
