@@ -109,8 +109,10 @@ GST_START_TEST (create_events)
   /* GAP */
   {
     GstClockTime ts = 0, dur = 0;
+    GstEvent *ignored G_GNUC_UNUSED;
 
-    ASSERT_CRITICAL (gst_event_new_gap (GST_CLOCK_TIME_NONE, GST_SECOND));
+    ASSERT_CRITICAL (ignored =
+        gst_event_new_gap (GST_CLOCK_TIME_NONE, GST_SECOND));
 
     event = gst_event_new_gap (90 * GST_SECOND, GST_SECOND);
     fail_if (event == NULL);
@@ -218,6 +220,7 @@ GST_START_TEST (create_events)
     GstSeekFlags flags;
     GstSeekType start_type, stop_type;
     gint64 start, stop;
+    GstClockTime trickmode_interval;
 
     event = gst_event_new_seek (0.5, GST_FORMAT_BYTES,
         GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_ACCURATE,
@@ -239,6 +242,22 @@ GST_START_TEST (create_events)
     fail_unless (stop_type == GST_SEEK_TYPE_NONE);
     fail_unless (stop == 0xdeadbeef);
 
+    gst_event_parse_seek_trickmode_interval (event, &trickmode_interval);
+    fail_unless_equals_uint64 (trickmode_interval, 0);
+
+    gst_event_set_seek_trickmode_interval (event, GST_SECOND);
+    gst_event_parse_seek_trickmode_interval (event, &trickmode_interval);
+    fail_unless_equals_uint64 (trickmode_interval, GST_SECOND);
+
+    gst_event_ref (event);
+    ASSERT_CRITICAL (gst_event_set_seek_trickmode_interval (event,
+            2 * GST_SECOND));
+    gst_event_unref (event);
+
+    gst_event_unref (event);
+
+    event = gst_event_new_flush_start ();
+    ASSERT_CRITICAL (gst_event_set_seek_trickmode_interval (event, GST_SECOND));
     gst_event_unref (event);
   }
 
@@ -412,13 +431,61 @@ GST_START_TEST (create_events)
     gst_event_unref (event);
     gst_event_unref (event2);
   }
+
+  /* GST_EVENT_INSTANT_RATE_CHANGE */
+  {
+    GstSeekFlags flags = GST_SEEK_FLAG_INSTANT_RATE_CHANGE;
+    GstSegmentFlags new_flags;
+    gdouble rate_multiplier;
+
+    event = gst_event_new_instant_rate_change (1.5, (GstSegmentFlags) flags);
+    fail_if (event == NULL);
+    fail_unless (GST_EVENT_TYPE (event) == GST_EVENT_INSTANT_RATE_CHANGE);
+    fail_unless (GST_EVENT_IS_DOWNSTREAM (event));
+    fail_unless (GST_EVENT_IS_STICKY (event));
+
+    gst_event_parse_instant_rate_change (event, &rate_multiplier, &new_flags);
+    fail_unless (rate_multiplier == 1.5);
+    fail_unless (new_flags == (flags & GST_SEGMENT_INSTANT_FLAGS));
+
+    gst_event_unref (event);
+  }
+
+  /* GST_EVENT_INSTANT_RATE_SYNC_TIME */
+  {
+    GstClockTime running_time;
+    GstClockTime upstream_running_time;
+    gdouble rate_multiplier;
+
+    running_time = 1 * GST_SECOND;
+    upstream_running_time = 2 * GST_SECOND;
+
+    event =
+        gst_event_new_instant_rate_sync_time (1.5, running_time,
+        upstream_running_time);
+    fail_if (event == NULL);
+    fail_unless (GST_EVENT_TYPE (event) == GST_EVENT_INSTANT_RATE_SYNC_TIME);
+    fail_unless (GST_EVENT_IS_UPSTREAM (event));
+
+    /* set some wrong values to check if the parse method overwrites them
+     * with the good values */
+    running_time = upstream_running_time = 3 * GST_SECOND;
+    gst_event_parse_instant_rate_sync_time (event, &rate_multiplier,
+        &running_time, &upstream_running_time);
+    fail_unless (rate_multiplier == 1.5);
+    fail_unless (running_time == 1 * GST_SECOND);
+    fail_unless (upstream_running_time == 2 * GST_SECOND);
+
+    gst_event_unref (event);
+  }
 }
 
 GST_END_TEST;
 
-static GTimeVal sent_event_time;
 static GstEvent *got_event_before_q, *got_event_after_q;
-static GTimeVal got_event_time;
+/* both time are in microseconds */
+static gint64 sent_event_time;
+static gint64 got_event_time;
 
 static GstPadProbeReturn
 event_probe (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
@@ -438,7 +505,7 @@ event_probe (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
         if (got_event_before_q != NULL)
           break;
         gst_event_ref ((GstEvent *) data);
-        g_get_current_time (&got_event_time);
+        got_event_time = g_get_real_time ();
         got_event_before_q = GST_EVENT (data);
         break;
       default:
@@ -453,7 +520,7 @@ event_probe (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
         if (got_event_after_q != NULL)
           break;
         gst_event_ref ((GstEvent *) data);
-        g_get_current_time (&got_event_time);
+        got_event_time = g_get_real_time ();
         got_event_after_q = GST_EVENT (data);
         break;
       default:
@@ -542,9 +609,8 @@ static void test_event
 
   event = gst_event_new_custom (type,
       gst_structure_new_empty ("application/x-custom"));
-  g_get_current_time (&sent_event_time);
-  got_event_time.tv_sec = 0;
-  got_event_time.tv_usec = 0;
+  sent_event_time = g_get_real_time ();
+  got_event_time = 0;
 
   signal_data_init (&data);
 
@@ -600,13 +666,6 @@ static void test_event
   signal_data_cleanup (&data);
 }
 
-static gint64
-timediff (GTimeVal * end, GTimeVal * start)
-{
-  return (end->tv_sec - start->tv_sec) * G_USEC_PER_SEC +
-      (end->tv_usec - start->tv_usec);
-}
-
 GST_START_TEST (send_custom_events)
 {
   /* Run some tests on custom events. Checking for serialisation and whatnot.
@@ -643,49 +702,42 @@ GST_START_TEST (send_custom_events)
 
   /* Upstream events */
   test_event (pipeline, GST_EVENT_CUSTOM_UPSTREAM, sinkpad, TRUE, srcpad);
-  fail_unless (timediff (&got_event_time,
-          &sent_event_time) < G_USEC_PER_SEC / 2,
+  fail_unless ((got_event_time - sent_event_time) < G_USEC_PER_SEC / 2,
       "GST_EVENT_CUSTOM_UP took too long to reach source: %"
-      G_GINT64_FORMAT " us", timediff (&got_event_time, &sent_event_time));
+      G_GINT64_FORMAT " us", got_event_time - sent_event_time);
 
   test_event (pipeline, GST_EVENT_CUSTOM_BOTH, sinkpad, TRUE, srcpad);
-  fail_unless (timediff (&got_event_time,
-          &sent_event_time) < G_USEC_PER_SEC / 2,
+  fail_unless ((got_event_time - sent_event_time) < G_USEC_PER_SEC / 2,
       "GST_EVENT_CUSTOM_BOTH took too long to reach source: %"
-      G_GINT64_FORMAT " us", timediff (&got_event_time, &sent_event_time));
+      G_GINT64_FORMAT " us", got_event_time - sent_event_time);
 
   test_event (pipeline, GST_EVENT_CUSTOM_BOTH_OOB, sinkpad, TRUE, srcpad);
-  fail_unless (timediff (&got_event_time,
-          &sent_event_time) < G_USEC_PER_SEC / 2,
+  fail_unless ((got_event_time - sent_event_time) < G_USEC_PER_SEC / 2,
       "GST_EVENT_CUSTOM_BOTH_OOB took too long to reach source: %"
-      G_GINT64_FORMAT " us", timediff (&got_event_time, &sent_event_time));
+      G_GINT64_FORMAT " us", got_event_time - sent_event_time);
 
   /* Out of band downstream events */
   test_event (pipeline, GST_EVENT_CUSTOM_DOWNSTREAM_OOB, srcpad, FALSE, srcpad);
-  fail_unless (timediff (&got_event_time,
-          &sent_event_time) < G_USEC_PER_SEC / 2,
+  fail_unless ((got_event_time - sent_event_time) < G_USEC_PER_SEC / 2,
       "GST_EVENT_CUSTOM_DS_OOB took too long to reach source: %"
-      G_GINT64_FORMAT " us", timediff (&got_event_time, &sent_event_time));
+      G_GINT64_FORMAT " us", got_event_time - sent_event_time);
 
   test_event (pipeline, GST_EVENT_CUSTOM_BOTH_OOB, srcpad, FALSE, srcpad);
-  fail_unless (timediff (&got_event_time,
-          &sent_event_time) < G_USEC_PER_SEC / 2,
+  fail_unless ((got_event_time - sent_event_time) < G_USEC_PER_SEC / 2,
       "GST_EVENT_CUSTOM_BOTH_OOB took too long to reach source: %"
-      G_GINT64_FORMAT " us", timediff (&got_event_time, &sent_event_time));
+      G_GINT64_FORMAT " us", got_event_time - sent_event_time);
 
   /* In-band downstream events are expected to take at least 1 second
    * to traverse the queue */
   test_event (pipeline, GST_EVENT_CUSTOM_DOWNSTREAM, srcpad, FALSE, srcpad);
-  fail_unless (timediff (&got_event_time,
-          &sent_event_time) >= G_USEC_PER_SEC / 2,
+  fail_unless ((got_event_time - sent_event_time) >= G_USEC_PER_SEC / 2,
       "GST_EVENT_CUSTOM_DS arrived too quickly for an in-band event: %"
-      G_GINT64_FORMAT " us", timediff (&got_event_time, &sent_event_time));
+      G_GINT64_FORMAT " us", got_event_time - sent_event_time);
 
   test_event (pipeline, GST_EVENT_CUSTOM_BOTH, srcpad, FALSE, srcpad);
-  fail_unless (timediff (&got_event_time,
-          &sent_event_time) >= G_USEC_PER_SEC / 2,
+  fail_unless ((got_event_time - sent_event_time) >= G_USEC_PER_SEC / 2,
       "GST_EVENT_CUSTOM_BOTH arrived too quickly for an in-band event: %"
-      G_GINT64_FORMAT " us", timediff (&got_event_time, &sent_event_time));
+      G_GINT64_FORMAT " us", got_event_time - sent_event_time);
 
   gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_NULL);
   gst_element_get_state (GST_ELEMENT (pipeline), NULL, NULL,
